@@ -12,7 +12,6 @@ import contextlib
 import errno
 import os
 import select
-import shutil
 import signal
 import socket
 import stat
@@ -26,10 +25,6 @@ import types
 from socket import AF_INET
 from socket import SOCK_DGRAM
 from socket import SOCK_STREAM
-try:
-    import ipaddress  # python >= 3.3
-except ImportError:
-    ipaddress = None
 
 import psutil
 
@@ -53,9 +48,7 @@ from psutil.tests import APPVEYOR
 from psutil.tests import call_until
 from psutil.tests import chdir
 from psutil.tests import check_connection_ntuple
-from psutil.tests import create_temp_executable_file
-from psutil.tests import decode_path
-from psutil.tests import encode_path
+from psutil.tests import create_exe
 from psutil.tests import enum
 from psutil.tests import get_test_subprocess
 from psutil.tests import get_winver
@@ -68,8 +61,7 @@ from psutil.tests import reap_children
 from psutil.tests import retry_before_failing
 from psutil.tests import RLIMIT_SUPPORT
 from psutil.tests import run_test_module_by_name
-from psutil.tests import safe_remove
-from psutil.tests import safe_rmdir
+from psutil.tests import safe_rmpath
 from psutil.tests import sh
 from psutil.tests import skip_on_access_denied
 from psutil.tests import skip_on_not_implemented
@@ -83,7 +75,6 @@ from psutil.tests import VALID_PROC_STATUSES
 from psutil.tests import wait_for_file
 from psutil.tests import wait_for_pid
 from psutil.tests import warn
-from psutil.tests import which
 from psutil.tests import WIN_VISTA
 
 
@@ -95,7 +86,7 @@ class TestProcess(unittest.TestCase):
     """Tests for psutil.Process class."""
 
     def setUp(self):
-        safe_remove(TESTFN)
+        safe_rmpath(TESTFN)
 
     def tearDown(self):
         reap_children()
@@ -106,7 +97,7 @@ class TestProcess(unittest.TestCase):
         self.assertEqual(psutil.Process(sproc.pid).pid, sproc.pid)
 
     def test_kill(self):
-        sproc = get_test_subprocess(wait=True)
+        sproc = get_test_subprocess()
         test_pid = sproc.pid
         p = psutil.Process(test_pid)
         p.kill()
@@ -116,7 +107,7 @@ class TestProcess(unittest.TestCase):
             self.assertEqual(sig, signal.SIGKILL)
 
     def test_terminate(self):
-        sproc = get_test_subprocess(wait=True)
+        sproc = get_test_subprocess()
         test_pid = sproc.pid
         p = psutil.Process(test_pid)
         p.terminate()
@@ -139,19 +130,17 @@ class TestProcess(unittest.TestCase):
             p = psutil.Process(sproc.pid)
             p.send_signal(sig)
             with mock.patch('psutil.os.kill',
-                            side_effect=OSError(errno.ESRCH, "")) as fun:
+                            side_effect=OSError(errno.ESRCH, "")):
                 with self.assertRaises(psutil.NoSuchProcess):
                     p.send_signal(sig)
-                assert fun.called
             #
             sproc = get_test_subprocess()
             p = psutil.Process(sproc.pid)
             p.send_signal(sig)
             with mock.patch('psutil.os.kill',
-                            side_effect=OSError(errno.EPERM, "")) as fun:
+                            side_effect=OSError(errno.EPERM, "")):
                 with self.assertRaises(psutil.AccessDenied):
                     psutil.Process().send_signal(sig)
-                assert fun.called
             # Sending a signal to process with PID 0 is not allowed as
             # it would affect every process in the process group of
             # the calling process (os.getpid()) instead of PID 0").
@@ -259,6 +248,8 @@ class TestProcess(unittest.TestCase):
                 self.assertLessEqual(percent, 100.0)
             else:
                 self.assertGreaterEqual(percent, 0.0)
+        with self.assertRaises(ValueError):
+            p.cpu_percent(interval=-1)
 
     def test_cpu_times(self):
         times = psutil.Process().cpu_times()
@@ -276,7 +267,7 @@ class TestProcess(unittest.TestCase):
     # try this with Python 2.7 and re-enable the test.
 
     @unittest.skipUnless(sys.version_info > (2, 6, 1) and not OSX,
-                         'os.times() is not reliable on this Python version')
+                         'os.times() broken on OSX + PY2.6.1')
     def test_cpu_times_2(self):
         user_time, kernel_time = psutil.Process().cpu_times()[:2]
         utime, ktime = os.times()[:2]
@@ -291,7 +282,7 @@ class TestProcess(unittest.TestCase):
             self.fail("expected: %s, found: %s" % (ktime, kernel_time))
 
     def test_create_time(self):
-        sproc = get_test_subprocess(wait=True)
+        sproc = get_test_subprocess()
         now = time.time()
         p = psutil.Process(sproc.pid)
         create_time = p.create_time()
@@ -307,17 +298,17 @@ class TestProcess(unittest.TestCase):
         # make sure returned value can be pretty printed with strftime
         time.strftime("%Y %m %d %H:%M:%S", time.localtime(p.create_time()))
 
-    @unittest.skipIf(WINDOWS, 'Windows only')
+    @unittest.skipUnless(POSIX, 'POSIX only')
     def test_terminal(self):
         terminal = psutil.Process().terminal()
         if sys.stdin.isatty():
             tty = os.path.realpath(sh('tty'))
             self.assertEqual(terminal, tty)
         else:
-            assert terminal, repr(terminal)
+            self.assertIsNone(terminal)
 
     @unittest.skipUnless(LINUX or BSD or WINDOWS,
-                         'not available on this platform')
+                         'platform not supported')
     @skip_on_not_implemented(only_if=LINUX)
     def test_io_counters(self):
         p = psutil.Process()
@@ -345,7 +336,7 @@ class TestProcess(unittest.TestCase):
         assert io2.read_bytes >= io1.read_bytes, (io1, io2)
 
     @unittest.skipUnless(LINUX or (WINDOWS and get_winver() >= WIN_VISTA),
-                         'Linux and Windows Vista only')
+                         'platform not supported')
     @unittest.skipIf(LINUX and TRAVIS, "unknown failure on travis")
     def test_ionice(self):
         if LINUX:
@@ -409,8 +400,7 @@ class TestProcess(unittest.TestCase):
             self.assertRaises(ValueError, p.ionice, 3)
             self.assertRaises(TypeError, p.ionice, 2, 1)
 
-    @unittest.skipUnless(LINUX and RLIMIT_SUPPORT,
-                         "only available on Linux >= 2.6.36")
+    @unittest.skipUnless(LINUX and RLIMIT_SUPPORT, "LINUX >= 2.6.36 only")
     def test_rlimit_get(self):
         import resource
         p = psutil.Process(os.getpid())
@@ -433,8 +423,7 @@ class TestProcess(unittest.TestCase):
                 self.assertGreaterEqual(ret[0], -1)
                 self.assertGreaterEqual(ret[1], -1)
 
-    @unittest.skipUnless(LINUX and RLIMIT_SUPPORT,
-                         "only available on Linux >= 2.6.36")
+    @unittest.skipUnless(LINUX and RLIMIT_SUPPORT, "LINUX >= 2.6.36 only")
     def test_rlimit_set(self):
         sproc = get_test_subprocess()
         p = psutil.Process(sproc.pid)
@@ -447,8 +436,7 @@ class TestProcess(unittest.TestCase):
         with self.assertRaises(ValueError):
             p.rlimit(psutil.RLIMIT_NOFILE, (5, 5, 5))
 
-    @unittest.skipUnless(LINUX and RLIMIT_SUPPORT,
-                         "only available on Linux >= 2.6.36")
+    @unittest.skipUnless(LINUX and RLIMIT_SUPPORT, "LINUX >= 2.6.36 only")
     def test_rlimit(self):
         p = psutil.Process()
         soft, hard = p.rlimit(psutil.RLIMIT_FSIZE)
@@ -467,8 +455,7 @@ class TestProcess(unittest.TestCase):
             p.rlimit(psutil.RLIMIT_FSIZE, (soft, hard))
             self.assertEqual(p.rlimit(psutil.RLIMIT_FSIZE), (soft, hard))
 
-    @unittest.skipUnless(LINUX and RLIMIT_SUPPORT,
-                         "only available on Linux >= 2.6.36")
+    @unittest.skipUnless(LINUX and RLIMIT_SUPPORT, "LINUX >= 2.6.36 only")
     def test_rlimit_infinity(self):
         # First set a limit, then re-set it by specifying INFINITY
         # and assume we overridden the previous limit.
@@ -483,8 +470,7 @@ class TestProcess(unittest.TestCase):
             p.rlimit(psutil.RLIMIT_FSIZE, (soft, hard))
             self.assertEqual(p.rlimit(psutil.RLIMIT_FSIZE), (soft, hard))
 
-    @unittest.skipUnless(LINUX and RLIMIT_SUPPORT,
-                         "only available on Linux >= 2.6.36")
+    @unittest.skipUnless(LINUX and RLIMIT_SUPPORT, "LINUX >= 2.6.36 only")
     def test_rlimit_infinity_value(self):
         # RLIMIT_FSIZE should be RLIM_INFINITY, which will be a really
         # big number on a platform with large file support.  On these
@@ -519,7 +505,7 @@ class TestProcess(unittest.TestCase):
             if thread._running:
                 thread.stop()
 
-    @unittest.skipUnless(WINDOWS, 'Windows only')
+    @unittest.skipUnless(WINDOWS, 'WINDOWS only')
     def test_num_handles(self):
         # a better test is done later into test/_windows.py
         p = psutil.Process()
@@ -557,19 +543,23 @@ class TestProcess(unittest.TestCase):
 
     @retry_before_failing()
     # see: https://travis-ci.org/giampaolo/psutil/jobs/111842553
-    @unittest.skipIf(OSX and TRAVIS, "")
+    @unittest.skipIf(OSX and TRAVIS, "fails on TRAVIS + OSX")
+    @skip_on_access_denied(only_if=OSX)
     def test_threads_2(self):
-        p = psutil.Process()
+        sproc = get_test_subprocess()
+        p = psutil.Process(sproc.pid)
         if OPENBSD:
             try:
                 p.threads()
             except psutil.AccessDenied:
                 raise unittest.SkipTest(
                     "on OpenBSD this requires root access")
-        self.assertAlmostEqual(p.cpu_times().user,
-                               p.threads()[0].user_time, delta=0.1)
-        self.assertAlmostEqual(p.cpu_times().system,
-                               p.threads()[0].system_time, delta=0.1)
+        self.assertAlmostEqual(
+            p.cpu_times().user,
+            sum([x.user_time for x in p.threads()]), delta=0.1)
+        self.assertAlmostEqual(
+            p.cpu_times().system,
+            sum([x.system_time for x in p.threads()]), delta=0.1)
 
     def test_memory_info(self):
         p = psutil.Process()
@@ -602,16 +592,19 @@ class TestProcess(unittest.TestCase):
             self.assertGreaterEqual(getattr(mem, name), 0)
 
     def test_memory_full_info(self):
+        total = psutil.virtual_memory().total
         mem = psutil.Process().memory_full_info()
         for name in mem._fields:
-            self.assertGreaterEqual(getattr(mem, name), 0)
+            value = getattr(mem, name)
+            self.assertGreaterEqual(value, 0, msg=(name, value))
+            self.assertLessEqual(value, total, msg=(name, value, total))
         if LINUX or WINDOWS or OSX:
-            self.assertGreater(mem.uss, 0)
+            mem.uss
         if LINUX:
-            self.assertGreater(mem.pss, 0)
+            mem.pss
             self.assertGreater(mem.pss, mem.uss)
 
-    @unittest.skipIf(OPENBSD or NETBSD, "not available on this platform")
+    @unittest.skipIf(OPENBSD or NETBSD, "platfform not supported")
     def test_memory_maps(self):
         p = psutil.Process()
         maps = p.memory_maps()
@@ -666,7 +659,7 @@ class TestProcess(unittest.TestCase):
             assert 0 <= ret <= 100, ret
 
     def test_is_running(self):
-        sproc = get_test_subprocess(wait=True)
+        sproc = get_test_subprocess()
         p = psutil.Process(sproc.pid)
         assert p.is_running()
         assert p.is_running()
@@ -676,7 +669,7 @@ class TestProcess(unittest.TestCase):
         assert not p.is_running()
 
     def test_exe(self):
-        sproc = get_test_subprocess(wait=True)
+        sproc = get_test_subprocess()
         exe = psutil.Process(sproc.pid).exe()
         try:
             self.assertEqual(exe, PYTHON)
@@ -693,11 +686,21 @@ class TestProcess(unittest.TestCase):
                 # We do not want to consider this difference in accuracy
                 # an error.
                 ver = "%s.%s" % (sys.version_info[0], sys.version_info[1])
-                self.assertEqual(exe.replace(ver, ''), PYTHON.replace(ver, ''))
+                try:
+                    self.assertEqual(exe.replace(ver, ''),
+                                     PYTHON.replace(ver, ''))
+                except AssertionError:
+                    # Tipically OSX. Really not sure what to do here.
+                    pass
+
+        subp = subprocess.Popen([exe, '-c', 'import os; print("hey")'],
+                                stdout=subprocess.PIPE)
+        out, _ = subp.communicate()
+        self.assertEqual(out.strip(), b'hey')
 
     def test_cmdline(self):
         cmdline = [PYTHON, "-c", "import time; time.sleep(60)"]
-        sproc = get_test_subprocess(cmdline, wait=True)
+        sproc = get_test_subprocess(cmdline)
         try:
             self.assertEqual(' '.join(psutil.Process(sproc.pid).cmdline()),
                              ' '.join(cmdline))
@@ -713,18 +716,20 @@ class TestProcess(unittest.TestCase):
                 raise
 
     def test_name(self):
-        sproc = get_test_subprocess(PYTHON, wait=True)
+        sproc = get_test_subprocess(PYTHON)
         name = psutil.Process(sproc.pid).name().lower()
         pyexe = os.path.basename(os.path.realpath(sys.executable)).lower()
         assert pyexe.startswith(name), (pyexe, name)
 
-    @unittest.skipIf(SUNOS, "doesn't work on Solaris")
+    # XXX
+    @unittest.skipIf(SUNOS, "broken on SUNOS")
     def test_prog_w_funky_name(self):
         # Test that name(), exe() and cmdline() correctly handle programs
         # with funky chars such as spaces and ")", see:
         # https://github.com/giampaolo/psutil/issues/628
-        funky_path = create_temp_executable_file('foo bar )')
-        self.addCleanup(safe_remove, funky_path)
+        funky_path = TESTFN + 'foo bar )'
+        create_exe(funky_path)
+        self.addCleanup(safe_rmpath, funky_path)
         cmdline = [funky_path, "-c",
                    "import time; [time.sleep(0.01) for x in range(3000)];"
                    "arg1", "arg2", "", "arg3", ""]
@@ -738,7 +743,7 @@ class TestProcess(unittest.TestCase):
         self.assertEqual(os.path.normcase(p.exe()),
                          os.path.normcase(funky_path))
 
-    @unittest.skipUnless(POSIX, 'posix only')
+    @unittest.skipUnless(POSIX, 'POSIX only')
     def test_uids(self):
         p = psutil.Process()
         real, effective, saved = p.uids()
@@ -751,7 +756,7 @@ class TestProcess(unittest.TestCase):
         if hasattr(os, "getresuid"):
             self.assertEqual(saved, os.getresuid()[2])
 
-    @unittest.skipUnless(POSIX, 'posix only')
+    @unittest.skipUnless(POSIX, 'POSIX only')
     def test_gids(self):
         p = psutil.Process()
         real, effective, saved = p.gids()
@@ -759,7 +764,7 @@ class TestProcess(unittest.TestCase):
         self.assertEqual(real, os.getgid())
         # os.geteuid() refers to "effective" uid
         self.assertEqual(effective, os.getegid())
-        # no such thing as os.getsuid() ("saved" uid), but starting
+        # no such thing as os.getsgid() ("saved" gid), but starting
         # from python 2.7 we have os.getresgid()[2]
         if hasattr(os, "getresuid"):
             self.assertEqual(saved, os.getresgid()[2])
@@ -824,19 +829,18 @@ class TestProcess(unittest.TestCase):
             p.username()
 
     def test_cwd(self):
-        sproc = get_test_subprocess(wait=True)
+        sproc = get_test_subprocess()
         p = psutil.Process(sproc.pid)
         self.assertEqual(p.cwd(), os.getcwd())
 
     def test_cwd_2(self):
         cmd = [PYTHON, "-c", "import os, time; os.chdir('..'); time.sleep(60)"]
-        sproc = get_test_subprocess(cmd, wait=True)
+        sproc = get_test_subprocess(cmd)
         p = psutil.Process(sproc.pid)
         call_until(p.cwd, "ret == os.path.dirname(os.getcwd())")
 
-    @unittest.skipUnless(WINDOWS or LINUX or FREEBSD,
-                         'not available on this platform')
-    @unittest.skipIf(LINUX and TRAVIS, "unknown failure on travis")
+    @unittest.skipUnless(WINDOWS or LINUX or FREEBSD, 'platform not supported')
+    @unittest.skipIf(LINUX and TRAVIS, "unreliable on TRAVIS")
     def test_cpu_affinity(self):
         p = psutil.Process()
         initial = p.cpu_affinity()
@@ -869,11 +873,12 @@ class TestProcess(unittest.TestCase):
         self.assertRaises(ValueError, p.cpu_affinity, invalid_cpu)
         self.assertRaises(ValueError, p.cpu_affinity, range(10000, 11000))
         self.assertRaises(TypeError, p.cpu_affinity, [0, "1"])
+        self.assertRaises(ValueError, p.cpu_affinity, [0, -1])
 
-    # TODO
-    @unittest.skipIf(BSD, "broken on BSD, see #595")
-    @unittest.skipIf(APPVEYOR,
-                     "can't find any process file on Appveyor")
+    # TODO: #595
+    @unittest.skipIf(BSD, "broken on BSD")
+    # can't find any process file on Appveyor
+    @unittest.skipIf(APPVEYOR, "unreliable on APPVEYOR")
     def test_open_files(self):
         # current process
         p = psutil.Process()
@@ -896,7 +901,7 @@ class TestProcess(unittest.TestCase):
 
         # another process
         cmdline = "import time; f = open(r'%s', 'r'); time.sleep(60);" % TESTFN
-        sproc = get_test_subprocess([PYTHON, "-c", cmdline], wait=True)
+        sproc = get_test_subprocess([PYTHON, "-c", cmdline])
         p = psutil.Process(sproc.pid)
 
         for x in range(100):
@@ -909,10 +914,10 @@ class TestProcess(unittest.TestCase):
         for file in filenames:
             assert os.path.isfile(file), file
 
-    # TODO
-    @unittest.skipIf(BSD, "broken on BSD, see #595")
-    @unittest.skipIf(APPVEYOR,
-                     "can't find any process file on Appveyor")
+    # TODO: #595
+    @unittest.skipIf(BSD, "broken on BSD")
+    # can't find any process file on Appveyor
+    @unittest.skipIf(APPVEYOR, "unreliable on APPVEYOR")
     def test_open_files_2(self):
         # test fd and path fields
         with open(TESTFN, 'w') as fileobj:
@@ -936,10 +941,8 @@ class TestProcess(unittest.TestCase):
 
     def compare_proc_sys_cons(self, pid, proc_cons):
         from psutil._common import pconn
-        sys_cons = []
-        for c in psutil.net_connections(kind='all'):
-            if c.pid == pid:
-                sys_cons.append(pconn(*c[:-1]))
+        sys_cons = [c[:-1] for c in psutil.net_connections(kind='all')
+                    if c.pid == pid]
         if FREEBSD:
             # on FreeBSD all fds are set to -1
             proc_cons = [pconn(*[-1] + list(x[1:])) for x in proc_cons]
@@ -1040,15 +1043,15 @@ class TestProcess(unittest.TestCase):
                                psutil.CONN_NONE,
                                ("all", "inet", "inet6", "udp", "udp6"))
 
-    @unittest.skipUnless(hasattr(socket, 'AF_UNIX'),
-                         'AF_UNIX is not supported')
+    @unittest.skipUnless(hasattr(socket, 'AF_UNIX'), 'AF_UNIX not supported')
     @skip_on_access_denied(only_if=OSX)
     def test_connections_unix(self):
         def check(type):
-            safe_remove(TESTFN)
+            safe_rmpath(TESTFN)
+            tfile = tempfile.mktemp(prefix=TESTFILE_PREFIX) if OSX else TESTFN
             sock = socket.socket(AF_UNIX, type)
             with contextlib.closing(sock):
-                sock.bind(TESTFN)
+                sock.bind(tfile)
                 cons = psutil.Process().connections(kind='unix')
                 conn = cons[0]
                 check_connection_ntuple(conn)
@@ -1056,7 +1059,7 @@ class TestProcess(unittest.TestCase):
                     self.assertEqual(conn.fd, sock.fileno())
                 self.assertEqual(conn.family, AF_UNIX)
                 self.assertEqual(conn.type, type)
-                self.assertEqual(conn.laddr, TESTFN)
+                self.assertEqual(conn.laddr, tfile)
                 if not SUNOS:
                     # XXX Solaris can't retrieve system-wide UNIX
                     # sockets.
@@ -1066,7 +1069,7 @@ class TestProcess(unittest.TestCase):
         check(SOCK_DGRAM)
 
     @unittest.skipUnless(hasattr(socket, "fromfd"),
-                         'socket.fromfd() is not availble')
+                         'socket.fromfd() not supported')
     @unittest.skipIf(WINDOWS or SUNOS,
                      'connection fd not available on this platform')
     def test_connection_fromfd(self):
@@ -1102,7 +1105,7 @@ class TestProcess(unittest.TestCase):
         if WINDOWS:
             psutil.CONN_DELETE_TCB
 
-    @unittest.skipUnless(POSIX, 'posix only')
+    @unittest.skipUnless(POSIX, 'POSIX only')
     def test_num_fds(self):
         p = psutil.Process()
         start = p.num_fds()
@@ -1117,7 +1120,7 @@ class TestProcess(unittest.TestCase):
         self.assertEqual(p.num_fds(), start)
 
     @skip_on_not_implemented(only_if=LINUX)
-    @unittest.skipIf(OPENBSD or NETBSD, "not reliable on Open/NetBSD")
+    @unittest.skipIf(OPENBSD or NETBSD, "not reliable on OPENBSD & NETBSD")
     def test_num_ctx_switches(self):
         p = psutil.Process()
         before = sum(p.num_ctx_switches())
@@ -1192,7 +1195,7 @@ class TestProcess(unittest.TestCase):
             self.assertEqual(len(c), len(set(c)))
 
     def test_suspend_resume(self):
-        sproc = get_test_subprocess(wait=True)
+        sproc = get_test_subprocess()
         p = psutil.Process(sproc.pid)
         p.suspend()
         for x in range(100):
@@ -1297,7 +1300,7 @@ class TestProcess(unittest.TestCase):
                     "NoSuchProcess exception not raised for %r, retval=%s" % (
                         name, ret))
 
-    @unittest.skipUnless(POSIX, 'posix only')
+    @unittest.skipUnless(POSIX, 'POSIX only')
     def test_zombie_process(self):
         def succeed_or_zombie_p_exc(fun, *args, **kwargs):
             try:
@@ -1309,6 +1312,7 @@ class TestProcess(unittest.TestCase):
         # Both of them are supposed to be freed / killed by
         # reap_children() as they are attributable to 'us'
         # (os.getpid()) via children(recursive=True).
+        unix_file = tempfile.mktemp(prefix=TESTFILE_PREFIX) if OSX else TESTFN
         src = textwrap.dedent("""\
         import os, sys, time, socket, contextlib
         child_pid = os.fork()
@@ -1324,11 +1328,11 @@ class TestProcess(unittest.TestCase):
                 else:
                     pid = bytes(str(os.getpid()), 'ascii')
                 s.sendall(pid)
-        """ % TESTFN)
+        """ % unix_file)
         with contextlib.closing(socket.socket(socket.AF_UNIX)) as sock:
             try:
                 sock.settimeout(GLOBAL_TIMEOUT)
-                sock.bind(TESTFN)
+                sock.bind(unix_file)
                 sock.listen(1)
                 pyrun(src)
                 conn, _ = sock.accept()
@@ -1393,46 +1397,36 @@ class TestProcess(unittest.TestCase):
 
     def test_pid_0(self):
         # Process(0) is supposed to work on all platforms except Linux
-        if 0 not in psutil.pids() and not OPENBSD:
+        if 0 not in psutil.pids():
             self.assertRaises(psutil.NoSuchProcess, psutil.Process, 0)
             return
 
+        # test all methods
         p = psutil.Process(0)
-        self.assertTrue(p.name())
-
-        if POSIX:
+        for name in psutil._as_dict_attrnames:
+            if name == 'pid':
+                continue
+            meth = getattr(p, name)
             try:
-                self.assertEqual(p.uids().real, 0)
-                self.assertEqual(p.gids().real, 0)
+                ret = meth()
             except psutil.AccessDenied:
                 pass
-
-            self.assertRaisesRegex(
-                ValueError, "preventing sending signal to process with PID 0",
-                p.send_signal, signal.SIGTERM)
-
-        self.assertIn(p.ppid(), (0, 1))
-        # self.assertEqual(p.exe(), "")
-        p.cmdline()
-        try:
-            p.num_threads()
-        except psutil.AccessDenied:
-            pass
-
-        try:
-            p.memory_info()
-        except psutil.AccessDenied:
-            pass
-
-        try:
-            if POSIX:
-                self.assertEqual(p.username(), 'root')
-            elif WINDOWS:
-                self.assertEqual(p.username(), 'NT AUTHORITY\\SYSTEM')
             else:
-                p.username()
-        except psutil.AccessDenied:
-            pass
+                if name in ("uids", "gids"):
+                    self.assertEqual(ret.real, 0)
+                elif name == "username":
+                    if POSIX:
+                        self.assertEqual(p.username(), 'root')
+                    elif WINDOWS:
+                        self.assertEqual(p.username(), 'NT AUTHORITY\\SYSTEM')
+                elif name == "name":
+                    assert name, name
+
+        if hasattr(p, 'rlimit'):
+            try:
+                p.rlimit(psutil.RLIMIT_FSIZE)
+            except psutil.AccessDenied:
+                pass
 
         p.as_dict()
 
@@ -1441,27 +1435,17 @@ class TestProcess(unittest.TestCase):
             self.assertTrue(psutil.pid_exists(0))
 
     def test_Popen(self):
-        # Popen class test
-        # XXX this test causes a ResourceWarning on Python 3 because
-        # psutil.__subproc instance doesn't get propertly freed.
-        # Not sure what to do though.
-        cmd = [PYTHON, "-c", "import time; time.sleep(60);"]
-        proc = psutil.Popen(cmd, stdout=subprocess.PIPE,
-                            stderr=subprocess.PIPE)
-        try:
+        with psutil.Popen([PYTHON, "-V"], stdout=subprocess.PIPE,
+                          stderr=subprocess.PIPE) as proc:
             proc.name()
+            proc.cpu_times()
             proc.stdin
-            self.assertTrue(hasattr(proc, 'name'))
-            self.assertTrue(hasattr(proc, 'stdin'))
             self.assertTrue(dir(proc))
             self.assertRaises(AttributeError, getattr, proc, 'foo')
-        finally:
-            proc.kill()
-            proc.wait()
-            self.assertIsNotNone(proc.returncode)
+        proc.wait()
 
     @unittest.skipUnless(hasattr(psutil.Process, "environ"),
-                         "environ not available")
+                         "platform not supported")
     def test_environ(self):
         self.maxDiff = None
         p = psutil.Process()
@@ -1485,7 +1469,7 @@ class TestProcess(unittest.TestCase):
         self.assertEqual(d, d2)
 
     @unittest.skipUnless(hasattr(psutil.Process, "environ"),
-                         "environ not available")
+                         "platform not supported")
     @unittest.skipUnless(POSIX, "posix only")
     def test_weird_environ(self):
         # environment variables can contain values without an equals sign
@@ -1502,8 +1486,9 @@ class TestProcess(unittest.TestCase):
             return execve("/bin/cat", argv, envp);
         }
         """)
-        path = create_temp_executable_file("x", c_code=code)
-        self.addCleanup(safe_remove, path)
+        path = TESTFN
+        create_exe(path, c_code=code)
+        self.addCleanup(safe_rmpath, path)
         sproc = get_test_subprocess([path],
                                     stdin=subprocess.PIPE,
                                     stderr=subprocess.PIPE)
@@ -1529,9 +1514,12 @@ class TestFetchAllProcesses(unittest.TestCase):
     def setUp(self):
         if POSIX:
             import pwd
-            pall = pwd.getpwall()
-            self._uids = set([x.pw_uid for x in pall])
-            self._usernames = set([x.pw_name for x in pall])
+            import grp
+            users = pwd.getpwall()
+            groups = grp.getgrall()
+            self.all_uids = set([x.pw_uid for x in users])
+            self.all_usernames = set([x.pw_name for x in users])
+            self.all_gids = set([x.gr_gid for x in groups])
 
     def test_fetch_all(self):
         valid_procs = 0
@@ -1642,20 +1630,21 @@ class TestFetchAllProcesses(unittest.TestCase):
 
     def uids(self, ret, proc):
         for uid in ret:
-            self.assertTrue(uid >= 0)
-            self.assertIn(uid, self._uids)
+            self.assertGreaterEqual(uid, 0)
+            self.assertIn(uid, self.all_uids)
 
     def gids(self, ret, proc):
         # note: testing all gids as above seems not to be reliable for
         # gid == 30 (nodoby); not sure why.
         for gid in ret:
-            self.assertTrue(gid >= 0)
-            # self.assertIn(uid, self.gids
+            if not OSX and not NETBSD:
+                self.assertGreaterEqual(gid, 0)
+                self.assertIn(gid, self.all_gids)
 
     def username(self, ret, proc):
         self.assertTrue(ret)
         if POSIX:
-            self.assertIn(ret, self._usernames)
+            self.assertIn(ret, self.all_usernames)
 
     def status(self, ret, proc):
         self.assertTrue(ret != "")
@@ -1704,8 +1693,12 @@ class TestFetchAllProcesses(unittest.TestCase):
             assert ret.peak_pagefile >= ret.pagefile, ret
 
     def memory_full_info(self, ret, proc):
+        total = psutil.virtual_memory().total
         for name in ret._fields:
-            self.assertGreaterEqual(getattr(ret, name), 0)
+            value = getattr(ret, name)
+            self.assertGreaterEqual(value, 0, msg=(name, value))
+            self.assertLessEqual(value, total, msg=(name, value, total))
+
         if LINUX:
             self.assertGreaterEqual(ret.pss, ret.uss)
 
@@ -1807,6 +1800,7 @@ class TestFetchAllProcesses(unittest.TestCase):
 # --- Limited user tests
 # ===================================================================
 
+
 if POSIX and os.getuid() == 0:
     class LimitedUserTestCase(TestProcess):
         """Repeat the previous tests by using a limited user.
@@ -1833,7 +1827,7 @@ if POSIX and os.getuid() == 0:
                 setattr(self, attr, types.MethodType(test_, self))
 
         def setUp(self):
-            safe_remove(TESTFN)
+            safe_rmpath(TESTFN)
             TestProcess.setUp(self)
             os.setegid(1000)
             os.seteuid(1000)
@@ -1860,19 +1854,27 @@ if POSIX and os.getuid() == 0:
 # --- Unicode tests
 # ===================================================================
 
+
 class TestUnicode(unittest.TestCase):
-    # See: https://github.com/giampaolo/psutil/issues/655
+    """
+    Make sure that APIs returning a string are able to handle unicode,
+    see: https://github.com/giampaolo/psutil/issues/655
+    """
+    uexe = TESTFN + 'èfile'
+    udir = TESTFN + 'èdir'
 
     @classmethod
     def setUpClass(cls):
-        cls.uexe = create_temp_executable_file('è')
-        cls.ubasename = os.path.basename(cls.uexe)
-        assert 'è' in cls.ubasename
+        safe_rmpath(cls.uexe)
+        safe_rmpath(cls.udir)
+        create_exe(cls.uexe)
+        os.mkdir(cls.udir)
 
     @classmethod
     def tearDownClass(cls):
         if not APPVEYOR:
-            safe_remove(cls.uexe)
+            safe_rmpath(cls.uexe)
+            safe_rmpath(cls.udir)
 
     def setUp(self):
         reap_children()
@@ -1883,16 +1885,17 @@ class TestUnicode(unittest.TestCase):
         subp = get_test_subprocess(cmd=[self.uexe])
         p = psutil.Process(subp.pid)
         self.assertIsInstance(p.name(), str)
-        self.assertEqual(os.path.basename(p.name()), self.ubasename)
+        self.assertEqual(p.exe(), self.uexe)
 
     def test_proc_name(self):
         subp = get_test_subprocess(cmd=[self.uexe])
         if WINDOWS:
+            # XXX: why is this like this?
             from psutil._pswindows import py2_strencode
             name = py2_strencode(psutil._psplatform.cext.proc_name(subp.pid))
         else:
             name = psutil.Process(subp.pid).name()
-        self.assertEqual(name, self.ubasename)
+        self.assertEqual(name, os.path.basename(self.uexe))
 
     def test_proc_cmdline(self):
         subp = get_test_subprocess(cmd=[self.uexe])
@@ -1901,14 +1904,12 @@ class TestUnicode(unittest.TestCase):
         self.assertEqual(p.cmdline(), [self.uexe])
 
     def test_proc_cwd(self):
-        tdir = os.path.realpath(tempfile.mkdtemp(prefix="psutil-è-"))
-        self.addCleanup(safe_rmdir, tdir)
-        with chdir(tdir):
+        with chdir(self.udir):
             p = psutil.Process()
             self.assertIsInstance(p.cwd(), str)
-            self.assertEqual(p.cwd(), tdir)
+            self.assertEqual(p.cwd(), self.udir)
 
-    @unittest.skipIf(APPVEYOR, "")
+    # @unittest.skipIf(APPVEYOR, "unreliable on APPVEYOR")
     def test_proc_open_files(self):
         p = psutil.Process()
         start = set(p.open_files())
@@ -1920,10 +1921,11 @@ class TestUnicode(unittest.TestCase):
             # see https://github.com/giampaolo/psutil/issues/595
             self.skipTest("open_files on BSD is broken")
         self.assertIsInstance(path, str)
-        self.assertEqual(os.path.normcase(path), os.path.normcase(self.uexe))
+        self.assertEqual(os.path.normcase(path),
+                         os.path.normcase(self.uexe))
 
     @unittest.skipUnless(hasattr(psutil.Process, "environ"),
-                         "environ not available")
+                         "platform not supported")
     def test_proc_environ(self):
         env = os.environ.copy()
         env['FUNNY_ARG'] = self.uexe
@@ -1936,134 +1938,19 @@ class TestUnicode(unittest.TestCase):
         self.assertEqual(p.environ()['FUNNY_ARG'], uexe)
 
     def test_disk_usage(self):
-        path = tempfile.mkdtemp(prefix='psutil', suffix='è')
-        psutil.disk_usage(path)
+        psutil.disk_usage(self.udir)
 
 
-class TestNonUnicode(unittest.TestCase):
-    """Test handling of non-utf8 data."""
-
-    @classmethod
-    def setUpClass(cls):
-        if PY3:
-            # Fix around https://bugs.python.org/issue24230
-            cls.temp_directory = tempfile.mkdtemp().encode('utf8')
-        else:
-            cls.temp_directory = tempfile.mkdtemp(suffix=b"")
-
-        # Return an executable that runs until we close its stdin.
-        if WINDOWS:
-            cls.test_executable = which("cmd.exe")
-        else:
-            cls.test_executable = which("cat")
-
-    @classmethod
-    def tearDownClass(cls):
-        shutil.rmtree(cls.temp_directory, ignore_errors=True)
-
-    def setUp(self):
-        reap_children()
-
-    tearDown = setUp
-
-    def copy_file(self, src, dst):
-        # A wrapper around shutil.copy() which is broken on py < 3.4
-        # when passed bytes paths.
-        with open(src, 'rb') as input_:
-            with open(dst, 'wb') as output:
-                output.write(input_.read())
-        shutil.copymode(src, dst)
-
-    def test_proc_exe(self):
-        funny_executable = os.path.join(self.temp_directory, b"\xc0\x80")
-        self.copy_file(self.test_executable, funny_executable)
-        self.addCleanup(safe_remove, funny_executable)
-        subp = get_test_subprocess(cmd=[decode_path(funny_executable)],
-                                   stdin=subprocess.PIPE,
-                                   stdout=subprocess.PIPE,
-                                   stderr=subprocess.STDOUT)
-        p = psutil.Process(subp.pid)
-        self.assertIsInstance(p.exe(), str)
-        self.assertEqual(encode_path(os.path.basename(p.exe())), b"\xc0\x80")
-        subp.communicate()
-        self.assertEqual(subp.returncode, 0)
-
-    def test_proc_name(self):
-        funny_executable = os.path.join(self.temp_directory, b"\xc0\x80")
-        self.copy_file(self.test_executable, funny_executable)
-        self.addCleanup(safe_remove, funny_executable)
-        subp = get_test_subprocess(cmd=[decode_path(funny_executable)],
-                                   stdin=subprocess.PIPE,
-                                   stdout=subprocess.PIPE,
-                                   stderr=subprocess.STDOUT)
-        p = psutil.Process(subp.pid)
-        self.assertEqual(encode_path(os.path.basename(p.name())), b"\xc0\x80")
-        subp.communicate()
-        self.assertEqual(subp.returncode, 0)
-
-    def test_proc_cmdline(self):
-        funny_executable = os.path.join(self.temp_directory, b"\xc0\x80")
-        self.copy_file(self.test_executable, funny_executable)
-        self.addCleanup(safe_remove, funny_executable)
-        subp = get_test_subprocess(cmd=[decode_path(funny_executable)],
-                                   stdin=subprocess.PIPE,
-                                   stdout=subprocess.PIPE,
-                                   stderr=subprocess.STDOUT)
-        p = psutil.Process(subp.pid)
-        self.assertEqual(p.cmdline(), [decode_path(funny_executable)])
-        subp.communicate()
-        self.assertEqual(subp.returncode, 0)
-
-    def test_proc_cwd(self):
-        funny_directory = os.path.realpath(
-            os.path.join(self.temp_directory, b"\xc0\x80"))
-        os.mkdir(funny_directory)
-        self.addCleanup(safe_rmdir, funny_directory)
-        with chdir(funny_directory):
-            p = psutil.Process()
-            self.assertIsInstance(p.cwd(), str)
-            self.assertEqual(encode_path(p.cwd()), funny_directory)
-
-    @unittest.skipIf(WINDOWS, "does not work on windows")
-    def test_proc_open_files(self):
-        funny_file = os.path.join(self.temp_directory, b"\xc0\x80")
-        p = psutil.Process()
-        start = set(p.open_files())
-        with open(funny_file, 'wb'):
-            new = set(p.open_files())
-        path = (new - start).pop().path
-        if BSD and not path:
-            # XXX
-            # see https://github.com/giampaolo/psutil/issues/595
-            self.skipTest("open_files on BSD is broken")
-        self.assertIsInstance(path, str)
-        self.assertIn(funny_file, encode_path(path))
-
-    @unittest.skipUnless(hasattr(psutil.Process, "environ"),
-                         "environ not available")
-    def test_proc_environ(self):
-        env = os.environ.copy()
-        funny_path = self.temp_directory
-        # ...otherwise subprocess.Popen fails with TypeError (it
-        # wants a string)
-        env['FUNNY_ARG'] = \
-            decode_path(funny_path) if WINDOWS and PY3 else funny_path
-        sproc = get_test_subprocess(env=env)
-        p = psutil.Process(sproc.pid)
-        self.assertEqual(
-            encode_path(p.environ()['FUNNY_ARG']), funny_path)
-
-    def test_disk_usage(self):
-        funny_directory = os.path.realpath(
-            os.path.join(self.temp_directory, b"\xc0\x80"))
-        os.mkdir(funny_directory)
-        self.addCleanup(safe_rmdir, funny_directory)
-        if WINDOWS and PY3:
-            # Python 3 on Windows is moving towards accepting unicode
-            # paths only:
-            # http://bugs.python.org/issue26330
-            funny_directory = decode_path(funny_directory)
-        psutil.disk_usage(funny_directory)
+class TestInvalidUnicode(TestUnicode):
+    """Test handling of invalid utf8 data."""
+    if PY3:
+        uexe = (TESTFN.encode('utf8') + b"f\xc0\x80").decode(
+            'utf8', 'surrogateescape')
+        udir = (TESTFN.encode('utf8') + b"d\xc0\x80").decode(
+            'utf8', 'surrogateescape')
+    else:
+        uexe = TESTFN + b"f\xc0\x80"
+        udir = TESTFN + b"d\xc0\x80"
 
 
 if __name__ == '__main__':
